package com.qhtx.eta.domain.facade.impl;

import com.google.common.collect.Lists;
import com.qhtx.eta.common.annotation.Facade;
import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.common.eunms.DataSourceType;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.domain.annotation.DomainTransDataSource;
import com.qhtx.eta.domain.api.ApiServiceHolder;
import com.qhtx.eta.domain.entity.DWIndexDTO;
import com.qhtx.eta.domain.entity.DWIndexDataDTO;
import com.qhtx.eta.domain.enums.ETAInterfaceEnum;
import com.qhtx.eta.domain.facade.ETAFacadeService;
import com.qhtx.eta.domain.service.DWIndexFrameService;
import com.qhtx.eta.domain.service.ETAPushDataService;
import com.qhtx.eta.infra.entity.dw.TEtaIndexPushTask;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @author mac
 */
@Service
@Facade
@Slf4j
public class ETAFacadeServiceImpl implements ETAFacadeService {

    @Resource
    private DWIndexFrameService dwIndexFrameService;

    @Resource
    private ETAPushDataService etaPushDataService;
    @Resource
    private ApiServiceHolder apiServiceHolder;

    @Override
    @DomainTransDataSource(dataSourceType = DataSourceType.DW)
    public void pushIndex(String indexCode) {
        DWIndexDTO dwIndexDTO = dwIndexFrameService.addSyncIndex(indexCode);

        CompletableFuture<Void> future = CompletableFuture.runAsync(() -> {
            List<DWIndexDataDTO> dataList = dwIndexFrameService.getIndexData(indexCode);
            dataList.sort(Comparator.comparingLong(o -> o.getDate().getTime()));
//            dataList.stream().sorted(Comparator.comparing(DWIndexDataDTO::getDate)).collect(Collectors.toList());
            dwIndexDTO.setDataList(dataList);
            //api数据发送
            if (dwIndexDTO.getDataList().size() > ETAConstants.ETA_PUSH_DATA_LIMIT) {
                log.info("数据量超过限制，分批推送");
                Lists.partition(dataList, ETAConstants.ETA_PUSH_DATA_CHUNK).forEach(list -> {
                    Date latestDataDate = list.stream().map(DWIndexDataDTO::getDate).max(Comparator.comparing(Date::getTime)).get();
                    //防止数据漏更新，先推送，再更新本地任务记录，下游做好冗余和幂等
                    DWIndexDTO dataDTO = new DWIndexDTO();
                    dataDTO.setIndexCode(dwIndexDTO.getIndexCode());
                    dataDTO.setFrequency(dwIndexDTO.getFrequency());
                    dataDTO.setIndexName(dwIndexDTO.getIndexName());
                    dataDTO.setRemark(dwIndexDTO.getRemark());
                    dataDTO.setSourceName(dwIndexDTO.getSourceName());
                    dataDTO.setUnit(dwIndexDTO.getUnit());
                    dataDTO.setDataList(list);
                      List<?> result = apiServiceHolder.runApi(ETAInterfaceEnum.PUSH_ETA_INDEX, dataDTO);
                    if (result == null) {
                        log.error("调用ETA接口失败，停止更新");
                        throw new ETAException(ErrorEnum.PUSH_TO_ETA_ERROR);
                    }
                    try {
                        Thread.sleep(500);
                    } catch (InterruptedException e) {
                        log.warn("线程中断异常");
                    }
                    TEtaIndexPushTask tEtaIndexPushTask = etaPushDataService.getETAPushTaskByIndexCode(dataDTO.getIndexCode());
                    if (tEtaIndexPushTask == null) {
                        log.error("未找到对应指标任务记录:{}", dataDTO.getIndexCode());
                        throw new ETAException(ErrorEnum.INDEX_UPDATE_ERROR);
                    }
                    tEtaIndexPushTask.setLatestDataDate(new SimpleDateFormat(ETAConstants.DATE_PATTERN).format(latestDataDate));
                    int currentCount = tEtaIndexPushTask.getDataCount() == null ? 0 : tEtaIndexPushTask.getDataCount();
                    tEtaIndexPushTask.setDataCount(currentCount + list.size());
                    tEtaIndexPushTask.setUpdateTime(new Date());
                    tEtaIndexPushTask.setLastPushTime(new Date());
                    etaPushDataService.updateTaskByIndexCode(tEtaIndexPushTask);
                });
            } else {
                apiServiceHolder.runApi(ETAInterfaceEnum.PUSH_ETA_INDEX, dwIndexDTO);
            }
        }).thenRun(() -> {
            log.info("同步指标任务表完成");
        });
        future.exceptionally(exception -> {
            log.error("同步指标任务表失败:", exception);
            return null;
        });

    }
}
